package com.tzld.piaoquan.recommend.server.service.rank.strategy;

import com.ctrip.framework.apollo.spring.annotation.ApolloJsonValue;
import com.tzld.piaoquan.recommend.server.model.Video;
import com.tzld.piaoquan.recommend.server.service.rank.RankParam;
import com.tzld.piaoquan.recommend.server.util.MathUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.math3.util.Pair;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

@Service
@Slf4j
public class RankStrategy4RegionMergeModelV566 extends RankStrategy4RegionMergeModelBasic {
    @ApolloJsonValue("${rank.score.merge.weightv566:}")
    private Map<String, Double> mergeWeight;

    @Autowired
    private RankStrategy4RegionMergeModelV563 modelV563Service;

    @Autowired
    private RankStrategy4RegionMergeModelV564 modelV564Service;

    @Override
    public List<Video> mergeAndRankRovRecall(RankParam param) {
        Map<String, Double> mergeWeight = this.mergeWeight != null ? this.mergeWeight : new HashMap<>(0);

        List<Video> result;
        String rosScoreKey;
        int baseRankId = mergeWeight.getOrDefault("baseRankId", 563D).intValue();
        if (564 == baseRankId) {
            rosScoreKey = "NorXGBScore";
            result = modelV564Service.mergeAndRankRovRecall(param);
        } else {
            rosScoreKey = "hasReturnRovScore";
            result = modelV563Service.mergeAndRankRovRecall(param);
        }
        if (null != result && !result.isEmpty()) {
            try {
                Integer versionCode = param.getVersionCode();
                int orderVersionCode = mergeWeight.getOrDefault("orderVersionCode", 1500D).intValue();
                if (null != versionCode && versionCode == orderVersionCode) {
                    int keepTopN = mergeWeight.getOrDefault("keepTopN", 1D).intValue();
                    boolean useRandFlag = mergeWeight.getOrDefault("useRandFlag", 0D).intValue() > 0;
                    boolean firstRovFlag = mergeWeight.getOrDefault("firstRovFlag", 1D).intValue() > 0;
                    double rovRandRate = mergeWeight.getOrDefault("rovRandRate", 0.5);
                    return resort(keepTopN, useRandFlag, firstRovFlag, rovRandRate, rosScoreKey, result);
                }
            } catch (Exception e) {
                log.error("diversity resort error", e);
            }
        }
        return result;
    }

    private List<Video> resort(int keepTopN, boolean useRandFlag, boolean firstRovFlag, double rovRandRate, String rosScoreKey, List<Video> videos) {
        Map<Long, Video> videoMap = new HashMap<>();
        List<Pair<Long, Double>> rovList = new ArrayList<>();
        List<Pair<Long, Double>> rosList = new ArrayList<>();
        for (Video video : videos) {
            long vid = video.getVideoId();
            videoMap.put(vid, video);

            Map<String, Double> scoreMap = video.getScoresMap();
            rovList.add(Pair.create(vid, scoreMap.getOrDefault("fmRov", 0d)));
            rosList.add(Pair.create(vid, scoreMap.getOrDefault(rosScoreKey, 0d)));
        }
        rovList.sort(Comparator.comparingDouble(o -> -o.getSecond()));
        rosList.sort(Comparator.comparingDouble(o -> -o.getSecond()));

        // top
        Set<Long> hit = new HashSet<>();
        List<Video> result = new ArrayList<>();
        for (int i = 0; i < keepTopN && i < videos.size(); i++) {
            Video video = videos.get(i);
            hit.add(video.getVideoId());
            result.add(video);
        }

        // alternate
        int rovIndex = 0;
        int rosIndex = 0;
        boolean flag = firstRovFlag;
        for (int i = keepTopN; i < videos.size(); i++) {
            if (useRandFlag) {
                flag = MathUtil.nextDouble(0, 1) < rovRandRate;
            }
            if (flag) {
                rovIndex = fillVideo(rovIndex, rovList, videoMap, hit, result);
            } else {
                rosIndex = fillVideo(rosIndex, rosList, videoMap, hit, result);
            }
            flag = !flag;
        }
        return result;
    }

    private int fillVideo(int start, List<Pair<Long, Double>> list, Map<Long, Video> videoMap, Set<Long> hit, List<Video> result) {
        for (int i = start; i < list.size(); i++) {
            start++;
            Pair<Long, Double> pair = list.get(i);
            long vid = pair.getFirst();
            if (!hit.contains(vid)) {
                Video video = videoMap.get(vid);
                hit.add(vid);
                result.add(video);
                break;
            }
        }
        return start;
    }
}
